<?php
function lhf_speedfactor_cwv() {
    global $wpdb;

    $table_name = $wpdb->prefix . 'lhf_sf_cwv';

    $url     = home_url();
    $api_key = get_option( 'lhf_speedfactor_lighthouse_api' );

    if ( (string) $api_key === '' ) {
        return;
    }

    $lhf_speedfactor_audit_http = (string) get_option( 'lhf_speedfactor_audit_http' );
    $lhf_speedfactor_audit_tls  = (string) get_option( 'lhf_speedfactor_audit_tls' );
    $lhf_speedfactor_audit_ipv4 = (string) get_option( 'lhf_speedfactor_audit_ipv4' );

    $curl = curl_init();

    curl_setopt( $curl, CURLOPT_URL, 'https://www.googleapis.com/pagespeedonline/v5/runPagespeed?key=AIzaSyAqC1bFupyhz0lSbobcQllrktegV3hU4Qk&screenshot=false&strategy=mobile&url=' . $url . '&category=performance' );
    curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
    curl_setopt( $curl, CURLOPT_ENCODING, '' );
    curl_setopt( $curl, CURLOPT_MAXREDIRS, 10 );
    curl_setopt( $curl, CURLOPT_TIMEOUT, 60 );

    if ( $lhf_speedfactor_audit_http === 'http1' ) {
        curl_setopt( $curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1 );
    } elseif ( $lhf_speedfactor_audit_http === 'http2' ) {
        curl_setopt( $curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_2_0 );
    }

    // TLS v1.3 or later (Added in 7.52.0)
    if ( $lhf_speedfactor_audit_tls === 'tls12' ) {
        curl_setopt( $curl, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2 );
    } elseif ( $lhf_speedfactor_audit_tls === 'tls13' ) {
        curl_setopt( $curl, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_3 );
    }

    if ( $lhf_speedfactor_audit_ipv4 === 'ipv4' ) {
        curl_setopt( $curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );
    }

    curl_setopt( $curl, CURLOPT_CUSTOMREQUEST, 'GET' );
    curl_setopt(
        $curl,
        CURLOPT_HTTPHEADER,
        [
            'Content-type: application/json',
            'cache-control: no-cache',
        ]
    );

    $response = curl_exec( $curl );
    $err      = curl_error( $curl );

    curl_close( $curl );

    if ( $err ) {
        return;
    } else {
        $response = (array) $response;
        $response = json_decode( $response[0], true );

        if ( ! $response ) {
            return;
        }

        if ( $response['loadingExperience'] ) {
            $metrics = $response['loadingExperience']['metrics'];

            $metrics_cls = 0;
            $metrics_fcp = 0;
            $metrics_lcp = 0;
            $metrics_fid = 0;

            if ( $metrics['CUMULATIVE_LAYOUT_SHIFT_SCORE'] ) {
                $metrics_cls = $metrics['CUMULATIVE_LAYOUT_SHIFT_SCORE']['percentile'];
            }
            if ( $metrics['FIRST_CONTENTFUL_PAINT_MS'] ) {
                $metrics_fcp = $metrics['FIRST_CONTENTFUL_PAINT_MS']['percentile'];
            }
            if ( $metrics['LARGEST_CONTENTFUL_PAINT_MS'] ) {
                $metrics_lcp = $metrics['LARGEST_CONTENTFUL_PAINT_MS']['percentile'];
            }
            if ( $metrics['FIRST_INPUT_DELAY_MS'] ) {
                $metrics_fid = $metrics['FIRST_INPUT_DELAY_MS']['percentile'];
            }

            /**
             * Save SEO score and performance results to database
             */
            $wpdb->query(
                $wpdb->prepare(
                    "INSERT INTO $table_name (
                        audit_first_contentful_paint,
                        audit_first_input_delay,
                        audit_cumulative_layout_shift,
                        audit_largest_contentful_paint,

                        audit_timestamp
                    ) VALUES (
                        %f,
                        %f,
                        %f,
                        %f,

                        %s
                    )",
                    [
                        $metrics_fcp,
                        $metrics_fid,
                        $metrics_cls,
                        $metrics_lcp,

                        gmdate( 'Y-m-d H:i:s' ),
                    ]
                )
            );
        }
    }
}
